// Copyright (C) 1997-2002 Alias|Wavefront, a division of Silicon Graphics Limited.
// 
// The information in this file is provided for the exclusive use of the
// licensees of Alias|Wavefront.  Such users have the right to use, modify,
// and incorporate this code into other products for purposes authorized
// by the Alias|Wavefront license agreement, without fee.
// 
// ALIAS|WAVEFRONT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
// EVENT SHALL ALIAS|WAVEFRONT BE LIABLE FOR ANY SPECIAL, INDIRECT OR
// CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
// DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
// TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
// PERFORMANCE OF THIS SOFTWARE.

//
//

//polyRawWriter.cpp

//Header File
//
#include "polyRawWriter.h"

//General Includes
//
#include <maya/MGlobal.h>
#include <maya/MIntArray.h>
#include <maya/MFnSet.h>
#include <maya/MItDependencyGraph.h>
#include <maya/MDagPath.h>
#include <maya/MFnMesh.h>
#include <maya/MPlug.h>
#include <iostream.h>
#include <time.h>

//Iterator Includes
//
#include <maya/MItMeshPolygon.h>

#include <maya/MItDependencyNodes.h>
#include <maya/MFnWeightGeometryFilter.h>
#include <maya/MFnGeometryFilter.h>
#include <maya/MItSelectionList.h>
#include <maya/MItGeometry.h>

#include <maya/MItDag.h> 
#include <maya/MFnIkJoint.h>

#include <maya/MTransformationMatrix.h>

#include <string.h>
#include <cstring>



//Macros
//
#define DELIMITER "\t"

#define CheckError(stat,msg) \
    if ( MStatus::kSuccess != stat ) {	\
		MGlobal::displayError(msg); \
		return MStatus::kFailure;   \
	}



polyRawWriter::polyRawWriter(const MDagPath& dagPath, MStatus& status)
//Summary:	creates and initializes an object of this class
//Args   :	dagPath - the DAG path of the current node
//		status - will be set to MStatus::kSuccess if the constructor was
//			 successful;  MStatus::kFailure otherwise
{
	fDagPath= new MDagPath(dagPath);
	fMesh	= new MFnMesh(*fDagPath, &status);


	tabs	= 0;

}


polyRawWriter::~polyRawWriter() 
//Summary:  deletes the objects created by this class
{
	if (NULL != fHeadUVSet) delete fHeadUVSet;
	if (NULL != fDagPath) delete fDagPath;
	if (NULL != fMesh) delete fMesh;

}


void polyRawWriter::doTabs(ostream &os  )
{                                                                                               
    int i;                                                                                      
    for ( i =0; i < tabs; i++ )
	    os << "\t";
//        fprintf(outputFile, " ");                                                               
}


MStatus polyRawWriter::extractGeometry() 
//Summary:	extracts main geometry as well as all UV sets and each set's
//			coordinates
{

	// MFnMesh*	fMesh;
	if (MStatus::kFailure == fMesh->getPoints(fVertexArray, MSpace::kWorld)) {
		MGlobal::displayError("MFnMesh::getPoints"); 
		return MStatus::kFailure;
	}

	if (MStatus::kFailure == fMesh->getFaceVertexColors(fColorArray)) {
		MGlobal::displayError("MFnMesh::getFaceVertexColors"); 
		return MStatus::kFailure;
	}

	if (MStatus::kFailure == fMesh->getNormals(fNormalArray, MSpace::kWorld)) {
		MGlobal::displayError("MFnMesh::getNormals"); 
		return MStatus::kFailure;
	}

	if (MStatus::kFailure == fMesh->getCurrentUVSetName(fCurrentUVSetName)) {
		MGlobal::displayError("MFnMesh::getCurrentUVSetName"); 
		return MStatus::kFailure;
	}

	//Have to make the path include the shape below it so that
	//we can determine if the underlying shape node is instanced.
	//By default, dag paths only include transform nodes.
	//
	fDagPath->extendToShape();

	//If the shape is instanced then we need to determine which
	//instance this path refers to.
	//
	int instanceNum = 0;
	if (fDagPath->isInstanced())
		instanceNum = fDagPath->instanceNumber();

	//Get the connected sets and members - these will be used to determine texturing of different
	//faces
	//
	if (!fMesh->getConnectedSetsAndMembers(instanceNum, fPolygonSets, fPolygonComponents, true)) {
		MGlobal::displayError("MFnMesh::getConnectedSetsAndMembers"); 
		return MStatus::kFailure;
	}

	MStringArray uvSetNames;

	if (MStatus::kFailure == fMesh->getUVSetNames(uvSetNames)) {
		MGlobal::displayError("MFnMesh::getUVSetNames"); 
		return MStatus::kFailure;
	}

	unsigned int uvSetCount = uvSetNames.length();
	unsigned int i;

	UVSet* currUVSet;

	for (i = 0; i < uvSetCount; i++ ) {
		if (0 == i) {
			currUVSet = new UVSet;
			fHeadUVSet = currUVSet;
		} else {
			currUVSet->next = new UVSet;
			currUVSet = currUVSet->next;
		}

		currUVSet->name = uvSetNames[i];
		currUVSet->next = NULL;

		// Retrieve the UV values
		//
		if (MStatus::kFailure == fMesh->getUVs(currUVSet->uArray, currUVSet->vArray, &currUVSet->name)) {
			return MStatus::kFailure;
		}
	}

	return MStatus::kSuccess;
}


MStatus polyRawWriter::writeShape(ostream& os) 
//Summary:	outputs the geometry of this polygonal mesh in raw text format
//Args   :	os - an output stream to write to
//Returns:  MStatus::kSuccess if the method succeeds
{
	MGlobal::displayInfo("Exporting " + fMesh->partialPathName());

//	os.form("Shape=  %s\n",fMesh->partialPathName.asChar());
	os << "Shape=  " << fMesh->partialPathName() << "\n";
	os.form("{\n");
	
	tabs++;

	if (MStatus::kFailure == outputVertices(os)) {
		return MStatus::kFailure;
	}

	if (MStatus::kFailure == outputUVs(os)) {
		return MStatus::kFailure;
	}

	if (MStatus::kFailure == outputSets(os)) {
		return MStatus::kFailure;
	}

	if (MStatus::kFailure == outputFaces(os)) {
		return MStatus::kFailure;
	}


	if (MStatus::kFailure == outputJoints(os)) {
		return MStatus::kFailure;
	}

	if (MStatus::kFailure == outputJointCluster(os)) {
		return MStatus::kFailure;
	}



//	if (MStatus::kFailure == outputVertexInfo(os)) {
//		return MStatus::kFailure;
//	}

//	if (MStatus::kFailure == outputNormals(os)) {
//		return MStatus::kFailure;
//	}

//	if (MStatus::kFailure == outputColors(os)) {
//		return MStatus::kFailure;
//	}
	os << "}\n\n";
	
	return MStatus::kSuccess;
}


MStatus polyRawWriter::outputFaces(ostream& os) 
//Summary:	outputs the vertex indices that comprise each face
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if all faces were outputted
//			MStatus::kFailure otherwise
{
	unsigned int faceCount = fMesh->numPolygons();
	if (0 == faceCount) {
		return MStatus::kFailure;
	}

	MStatus status;
	MIntArray indexArray;

	UVSet* currUVSet;
	currUVSet = fHeadUVSet;
	
	int uvID;


	doTabs(os);
	os.form("Faces=  %d\n",faceCount);
	doTabs(os);
	os << "{\n";  
	
	tabs++;
	unsigned int i;
	for (i = 0; i < faceCount; i++) {

		doTabs(os);	
		os.form( "%d\t",i );

		unsigned int indexCount = fMesh->polygonVertexCount(i, &status);

		if (MStatus::kFailure == status) {
			MGlobal::displayError("MFnMesh::polygonVertexCount");
			return MStatus::kFailure;
		}

		status = fMesh->getPolygonVertices (i, indexArray);

		if (MStatus::kFailure == status) {
			MGlobal::displayError("MFnMesh::getPolygonVertices");
			return MStatus::kFailure;
		}

		unsigned int j;
		for (j = 0; j < indexCount; j++) 
			os << indexArray[j] << " ";
		
		os << DELIMITER;

		for (j = 0; j < indexCount; j++) {
		    status = fMesh->getPolygonUVid(i, j, uvID, &currUVSet->name);
		    os << uvID << " ";
		    // os.form( "%6f %6f %6f\n",xPos, yPos, zPos );
		}


		os << "\n";
	}
	tabs--;
	doTabs(os);	
	os << "}\n\n";

	return MStatus::kSuccess;
}


MStatus polyRawWriter::outputJoints(ostream& os) 
{
	float		xPos, yPos, zPos;
	double		Orient[3];
	
	MString		path, templatePath;

	MPlug		plug;
	MPlugArray	plugArray;
	
	MStatus		status;
	MObject		joint;

	MDagPath	dagPath;
	
	MItDag::TraversalType traversalType = MItDag::kDepthFirst;

	MFn::Type	filter = MFn::kJoint;
		

	unsigned int jcCount = 0;
	
	MItDag dagIter1( traversalType, filter, &status );
	CheckError(status,"Iterator constructor problems!" ); // return GE_WRITE_FILE_ERROR;

	
	// To get Joints number!!!
	for ( ; !dagIter1.isDone(); dagIter1.next() )	    jcCount++; 

	if (jcCount==0)		return MStatus::kSuccess;


	doTabs(os);
	os.form("Joint=  %d\n", jcCount);
	doTabs(os);
	os << "{\n";  
	
	MItDag dagIter( traversalType, filter, &status );
	//   ʱȭ Ǵ± 
//	dagIter.reset();

		
	for ( ; !dagIter.isDone(); dagIter.next() )
    	{
		status = dagIter.getPath( dagPath );
		CheckError(status,"DagPath construction problems in joint...");

		joint = dagPath.node( &status );

		
		MFnIkJoint fnJoint( joint, &status );
		CheckError(status,"MFnIkJoint construction problems in joint...");

		tabs++;
		doTabs(os);
		os.form(" %s \n", fnJoint.name().asChar());
//
//   ϴ  ̰ 
//    ΰΰ . 
// false ? or true ?
//
		plug = fnJoint.findPlug( "translateX", &status );
		status = plug.getValue( xPos );
		plug = fnJoint.findPlug( "translateY", &status );
		status = plug.getValue( yPos );
		plug = fnJoint.findPlug( "translateZ", &status );
		status = plug.getValue( zPos );
		tabs++;
		doTabs(os);
		os.form( "%6f %6f %6f\n",xPos, yPos, zPos );

		plug = fnJoint.findPlug( "rotateX", &status );
		status = plug.getValue( xPos );
		plug = fnJoint.findPlug( "rotateY", &status );
		status = plug.getValue( yPos );
		plug = fnJoint.findPlug( "rotateZ", &status );
		status = plug.getValue( zPos );
		doTabs(os);
		os.form( "%6f %6f %6f\n",xPos, yPos, zPos );


		MTransformationMatrix::RotationOrder rotOrder;
		rotOrder=MTransformationMatrix::kXYZ;
	
	
		status= fnJoint.getOrientation(Orient,rotOrder);
		CheckError(status,"fnJoint.getOrientation error...");
		
		doTabs(os);
		os.form( "%6f %6f %6f\n",(float)Orient[0], (float)Orient[1], (float)Orient[2] );
		tabs--;
		tabs--;
	}

	doTabs(os);
	os << "}\n\n";


	MItDag dagIter3( traversalType, filter, &status );
	doTabs(os);
	os.form("JointHierachy \n");
	doTabs(os);
	os << "{\n";  
	
	unsigned child_n,i,n;
	MObject		child;
	MObject		next;
	MObject		current;
	MDagPath	dagCPath;

	tabs++;

	
	for ( ; !dagIter3.isDone(); dagIter3.next() )
    	{
				
		current= dagIter3.item(&status);
		status = dagIter3.getPath( dagPath );
		CheckError(status,"DagPath construction problems in joint...");

	
		joint = dagPath.node( &status );
		child_n = dagPath.childCount( &status ); //unsigned
	
		n=0;
		for(i=0;i<child_n;i++){
			child = dagPath.child(i,&status);
			if(child.apiType()==filter){
				n++;		
			}
		}

		MFnIkJoint fnJoint( joint, &status );
		CheckError(status,"MFnIkJoint1 construction problems in joint...");
		doTabs(os);
		os.form("%s\t%d\n", fnJoint.name().asChar(),n);

		 

	///	dagIter3.reset(joint,traversalType, filter); 
		tabs++;
		
		for(i=0;i<child_n;i++){
			child = dagPath.child(i,&status);
			if(child.apiType()==filter){
				MFnIkJoint fnJoint( child, &status );
				CheckError(status,"MFnIkJoint2 construction problems in joint...");
				doTabs(os);
				os.form(" %s \n", fnJoint.name().asChar());
			}
		} 
		tabs--;

//		dagIter3.reset(current,traversalType, filter); 
	}
		
	tabs--;	
	doTabs(os);
	os << "}\n\n";


	return MStatus::kSuccess;

}




MObject polyRawWriter::jointForCluster(MObject& jointCluster) const
//
// Given a jointCluster, return the joint that drives it
//
{
	MObject result;
	MFnDependencyNode fnNode(jointCluster);
	
	MObject attrJoint = fnNode.attribute("matrix");

	MPlug pJointPlug(jointCluster,attrJoint);
	MPlugArray conns;

	if (pJointPlug.connectedTo(conns,true,false)) {
		result = conns[0].node();
	}
	
	return result;
}



MStatus polyRawWriter::outputJointCluster(ostream& os) 
{
	MStatus stat;

	unsigned int jcCount = 0;

	// Iterate through graph and search for jointCluster nodes
	MItDependencyNodes iter( MFn::kJointCluster);


	// To get Joints number!!!
	for ( ; !iter.isDone(); iter.next() )    
	    jcCount++; 

//	if (jcCount==0)
//		return MStatus::kSuccess;

	doTabs(os);
	os.form("JointCluster=  %d\n", jcCount);
	doTabs(os);
	os << "{\n";  
	

	iter.reset();
	
	tabs++;
	for ( ; !iter.isDone(); iter.next() ) {
	// Joint 1 , 2, 3 , 4 ...
	//
	    MObject object = iter.item();
	    MFnWeightGeometryFilter jointCluster(object);

	    // get the joint driving this cluster
	    MObject joint = jointForCluster(object);

	    if (joint.isNull()) {
	    	MGlobal::displayError("Joint is not attached to cluster.");
	    	continue;
	    }

		MObject deformSet = jointCluster.deformerSet(&stat);
		CheckError(stat,"Error getting deformer set.");
			
		MFnSet setFn(deformSet, &stat);	//need the fn to get the members
		CheckError(stat,"Error getting deformer set fn.");
		
		MSelectionList clusterSetList;

		//get all the members
		//
		stat = setFn.getMembers(clusterSetList, true);
		CheckError(stat,"Could not make member list with getMembers.");

		// print out the name of joint and the number of associated skins
		//
		MFnDependencyNode fnJoint(joint);
	
		//fprintf(file,"%s %d\n", fnJoint.name().asChar(), clusterSetList.length());

		doTabs(os);
		os << fnJoint.name().asChar() ;
		
		if( clusterSetList.length()>1 ){
		//	stat=false;
				CheckError(stat,"clusterSetlist.lenth!!");
			os.form("clusterSetlist.lenth!!\n");
		}
		
		tabs++;
		for (size_t kk = 0; kk < clusterSetList.length(); ++kk) {
		    MObject joint;
		    MDagPath skinpath;
		    MObject components;
		    MFloatArray weights;

		    clusterSetList.getDagPath(kk,skinpath,components);
		    jointCluster.getWeights(skinpath,components,weights);

		// print out the path name of the skin & the weight count
		    os << " " << (float)weights.length() << "\n";

		// loop through the components and print their index and weight
		    unsigned counter =0;
		    MItGeometry gIter(skinpath,components);
		
		    doTabs(os);
		    for (; !gIter.isDone() && counter < weights.length() ; gIter.next()) 
		    {
    			   // os << gIter.index() << " " << weights[counter] << "  ";
    			    os.form(" %d", gIter.index());
			    counter++;
		    }
		    
    		    os << "\n";
		}
		tabs--;
		jcCount++;
	}



	tabs--;
	doTabs(os);
	os << "}\n\n";
	
	return MStatus::kSuccess;
}


MStatus polyRawWriter::outputVertices(ostream& os) 
//Summary:	outputs all vertex coordinates
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if all vertex coordinates were outputted
//			MStatus::kFailure otherwise
{
	unsigned int vertexCount = fVertexArray.length();
	unsigned i;

	if (0 == vertexCount) {
		return MStatus::kFailure;
	}

	//sync_with_stdio(true);

	doTabs(os);
	os.form("verteces= %d\n",vertexCount);

	doTabs(os);
	os << "{\n"; //Format:  Vertex|(x, y, z)\n";
	tabs++;

	for (i = 0; i < vertexCount; i++) {
		doTabs(os);
		os.form("%d\t %6f  %6f  %6f\n", i ,fVertexArray[i].x
		    ,fVertexArray[i].y , fVertexArray[i].z );
	}
	
	tabs--;
	doTabs(os);
	os << "}\n\n";
	

	return MStatus::kSuccess;
}


MStatus polyRawWriter::outputVertexInfo(ostream& os) 
//Summary:	outputs the per face per vertex information such as normal, color, and uv set
//			indices
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if all per face per vertex information was outputted
//			MStatus::kFailure otherwise
{
	unsigned int faceCount = fMesh->numPolygons();
	unsigned i, j, indexCount;

	MStatus status;
	MIntArray indexArray;

	//output the header
	os << "Vertex Info:\n";
	os << "Face|fVtexIdx|vtxIdx|normalIndex|colorIndex|";
	
	//Add each uv set to the header
	UVSet* currUVSet;
	for (currUVSet = fHeadUVSet; currUVSet != NULL; currUVSet = currUVSet->next) {
		os << "| UV_" << currUVSet->name;
	}
	os << "\n";


	MIntArray normalIndexArray;
	int colorIndex, uvID;

	for (i = 0; i < faceCount; i++) {

		indexCount = fMesh->polygonVertexCount(i, &status);
		if (MStatus::kFailure == status) {
			MGlobal::displayError("MFnMesh::polygonVertexCount");
			return MStatus::kFailure;
		}

		status = fMesh->getPolygonVertices (i, indexArray);
		if (MStatus::kFailure == status) {
			MGlobal::displayError("MFnMesh::getPolygonVertices");
			return MStatus::kFailure;
		}

		status == fMesh->getFaceNormalIds (i, normalIndexArray);
		if (MStatus::kFailure == status) {
			MGlobal::displayError("MFnMesh::getFaceNormalIds");
			return MStatus::kFailure;
		}

		for (j = 0; j < indexCount; j++) {
			status = fMesh->getFaceVertexColorIndex(i, j, colorIndex);

			//output the face, face vertex index, vertex index, normal index, color index
			//for the current vertex on the current face
			os << i << DELIMITER << j << DELIMITER << indexArray[j] << DELIMITER
			   << normalIndexArray[j] << DELIMITER << colorIndex << DELIMITER;

		//output each uv set index for the current vertex on the current face
			for (currUVSet = fHeadUVSet; currUVSet != NULL; currUVSet = currUVSet->next) {
				status = fMesh->getPolygonUVid(i, j, uvID, &currUVSet->name);
				if (MStatus::kFailure == status) {
					MGlobal::displayError("MFnMesh::getPolygonUVid");
					return MStatus::kFailure;
				}
				os << DELIMITER << uvID;
			}
			os << "\n";
		}

		os << "\n";
	}
	os << "\n";

	return MStatus::kSuccess;
}


MStatus polyRawWriter::outputNormals(ostream& os) 
//Summary:	outputs the normals for this polygonal mesh
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if all normals were outputted
//			MStatus::kFailure otherwise
{
	unsigned int normalCount = fNormalArray.length();
	if (0 == normalCount) {
		return MStatus::kFailure;
	}

	os << "Normals:  " << normalCount << "\n";
	os << "Format:  Index|[x, y, z]\n";

	unsigned int i;
	for (i = 0; i < normalCount; i++) {
		os << i << DELIMITER << "["
		   << fNormalArray[i].x << ", "
		   << fNormalArray[i].y << ", "
		   << fNormalArray[i].z << "]\n";
	}
	os << "\n\n";

	return MStatus::kSuccess;
}


MStatus polyRawWriter::outputColors(ostream& os) 
//Summary:	outputs the colors for this polygonal mesh
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if all colors were outputted
//			MStatus::kFailure otherwise
{
	unsigned int colorCount = fColorArray.length();
	if (0 == colorCount) {
		return MStatus::kFailure;
	}

	os << "Colors:  " << colorCount << "\n";
	//os << "Format:  Index|R G B A\n";
	
	unsigned int i;
	for (i = 0; i < colorCount; i++) {
		os << i << DELIMITER
		   << fColorArray[i].r << " "
		   << fColorArray[i].g << " "
		   << fColorArray[i].b << " "
		   << fColorArray[i].a << "\n";
	}
	os << "\n\n";

	return MStatus::kSuccess;
}


MStatus polyRawWriter::outputUVs(ostream& os) 
//Summary:	for each UV Set, outputs all uv coordinates
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if UV coordinates for all UV sets were outputted
//			MStatus::kFailure otherwise
{
	UVSet* currUVSet;
	unsigned int i, uvCount;
	for (currUVSet = fHeadUVSet; currUVSet != NULL; currUVSet = currUVSet->next) {
		
//		if (currUVSet->name == fCurrentUVSetName) os << "Current ";
		uvCount = currUVSet->uArray.length();

		doTabs(os);
		os << "UV_" << currUVSet->name << "  " << uvCount << "\n";
		
		doTabs(os);
		os << "{\n";
		tabs++;

		for (i = 0; i < uvCount; i++) {
		    doTabs(os);
		    os.form("%d\t %6f   %6f\n",i,currUVSet->uArray[i], currUVSet->vArray[i]);
//		    os << i << DELIMITER << currUVSet->uArray[i] << "\t" << currUVSet->vArray[i] << "\n";
		}
		tabs--;
		doTabs(os);
		os << "}\n";
	}
	os << "\n";
	return MStatus::kSuccess;
}

char *basename(const char *filename)
{
  char *p = strrchr (filename, '/');
  return p ? p + 1 : (char *) filename;
}



MStatus polyRawWriter::outputSingleSet(ostream& os, MString setName, MIntArray faces, MString textureName)
//Summary:	outputs this mesh's sets and each sets face components, and any 
//			associated texture
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if set information was outputted
//			MStatus::kFailure otherwise
{
	unsigned int i;
	unsigned int faceCount = faces.length();
	char	 name[64];

	strcpy(name,textureName.asChar());
	tabs++;
	doTabs(os);
	os.form("File= %s \n",basename(name));
	
	doTabs(os);
	os.form("Faces= %d\n",faceCount);

	doTabs(os);
	for (i = 0; i < faceCount; i++) { 
		os.form("%d ",faces[i]);
	}
	os << "\n";

	if (textureName == "") {
		textureName = "none";
	} 
//	doTabs(os);
	tabs--;

	return MStatus::kSuccess;
}


MObject polyRawWriter::findShader(const MObject& setNode) 
//Summary:	finds the shading node for the given shading group set node
//Args   :	setNode - the shading group set node
//Returns:  the shader node for setNode if found;
//			MObject::kNullObj otherwise
{

	MFnDependencyNode fnNode(setNode);
	MPlug shaderPlug = fnNode.findPlug("surfaceShader");
			
	if (!shaderPlug.isNull()) {			
		MPlugArray connectedPlugs;

		//get all the plugs that are connected as the destination of this 
		//surfaceShader plug so we can find the surface shaderNode
		//
		MStatus status;
		shaderPlug.connectedTo(connectedPlugs, true, false, &status);
		if (MStatus::kFailure == status) {
			MGlobal::displayError("MPlug::connectedTo");
			return MObject::kNullObj;
		}

		if (1 != connectedPlugs.length()) {
			MGlobal::displayError("Error getting shader for: " + fMesh->partialPathName());
		} else {
			return connectedPlugs[0].node();
		}
	}
	
	return MObject::kNullObj;
}




MStatus polyRawWriter::outputSets(ostream& os)
//Summary:	outputs this mesh's sets and each sets face components, and any 
//			associated texture
//Args   :	os - an output stream to write to
//Returns:	MStatus::kSuccess if set information was outputted
//			MStatus::kFailure otherwise
{
	MStatus status;

	//if there is more than one set, the last set simply consists of all 
	//polygons, so we won't include it
	//
	unsigned int setCount = fPolygonSets.length();

	if (setCount > 1) {
		setCount--;
	}

	MIntArray faces;
	

	unsigned int i;

	// TEXTURE  ΰ쿡 
	doTabs(os);
	os.form("Texture= %d \n",setCount);
	
	doTabs(os);
	os.form("{\n");
	
/*	int t_n;

	strcpy(name,textureName.asChar());
	os.form("Texture= %d \n\n",);
	
	os.form("Faces= %d\n",faceCount);
	for (i = 0; i < faceCount; i++) { 
		os.form("%d ",faces[i]);
	}
	os << "\n";

	if (textureName == "") {
		textureName = "none";
	} 
	doTabs(os); */



	for (i = 0; i < setCount; i++ ) {
		MObject set = fPolygonSets[i];
		MObject comp = fPolygonComponents[i];
		MFnSet fnSet(set, &status);
		if (MS::kFailure == status) {
			MGlobal::displayError("MFnSet::MFnSet");
            		continue;
        	}

        	//Make sure the set is a polygonal set.  If not, continue.
		MItMeshPolygon itMeshPolygon(*fDagPath, comp, &status);

		if ((MS::kFailure == status)) {
			MGlobal::displayError("MItMeshPolygon::MItMeshPolygon");
            		continue;
		}

		//add the current set's face indices to the faces array
		//
		faces.setLength(itMeshPolygon.count());

		unsigned int j = 0;
		for (itMeshPolygon.reset(); !itMeshPolygon.isDone(); itMeshPolygon.next()) {
			faces[j++] = itMeshPolygon.index();
		}

		//Find the texture that is applied to this set.  First, get the
		//shading node connected to the set.  Then, if there is an input
		//attribute called "color", search upstream from it for a texture
		//file node.
		//
		MObject shaderNode = findShader(set);
		if (MObject::kNullObj == shaderNode) {
			continue;
		}

		MPlug colorPlug = MFnDependencyNode(shaderNode).findPlug("color", &status);
		if (MS::kFailure == status) {
			MGlobal::displayError("MFnDependencyNode::findPlug");
			continue;
		}

		MItDependencyGraph itDG(colorPlug, MFn::kFileTexture,
								MItDependencyGraph::kUpstream, 
								MItDependencyGraph::kBreadthFirst,
								MItDependencyGraph::kNodeLevel, 
								&status);

		if (MS::kFailure == status) {
			MGlobal::displayError("MItDependencyGraph::MItDependencyGraph");
			continue;
		}

		//disable automatic pruning so that we can locate a specific plug 
		itDG.disablePruningOnFilter();

		//If no texture file node was found, pass in an empty string as the texture filename 
		//so that color information is outputted instead
		//


		MString textureName("");
		if (itDG.isDone()) {
			if (MStatus::kFailure == outputSingleSet(os, MString(fnSet.name()), faces, textureName)) {
				return MStatus::kFailure;
			}
		//otherwise retrieve the filename and pass it in to output texture information
		//
		} else {
			MObject textureNode = itDG.thisNode();
			MPlug filenamePlug = MFnDependencyNode(textureNode).findPlug("fileTextureName");
			filenamePlug.getValue(textureName);
			if (MStatus::kFailure == outputSingleSet(os, MString(fnSet.name()), faces, textureName)) 
			{
				return MStatus::kFailure;
			}

		}
	}
	
//	tabs--;
	doTabs(os);
	os <<"}\n\n";

	return MStatus::kSuccess;
}
